REBOL [
	Title: "Installer GUI"
	Rights: "Copyright REBOL Technologies 1997-2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

;do %utils.r
;do %register.r
;do %associate.r
;do %links.r

install-rebol: does [
	install-flag: true
	context installer-gui
]

uninstall-rebol: does [
	install-flag: false
	context installer-gui
]

installer-gui: [

reg: any [registered? "REBOL" form system/product []]
prog-path: any [reg/2 default-prog-dir?]
data-path: any [reg/3 default-data-dir?]

panelize: func [face] [
	; Commonly done for sub-panels:
	face/offset: 0x0
	face/color: none
	face
]

;-- Standard sizes:
s1: 133x280
s2: 317x280
s3: s1 + s2 * 1x0 + 0x40
sx: s2/x - 24

;-- Local variables:
f1: f2:
ck0: ck1: ck2: ck3: ckx: ckd:
panel-face:
back-btn:
next-panel:
err:
start-dir:
	none
panel-history: []
quick-install: true

l1: copy logo.gif
l1/alpha: 150
l2: copy logo.gif
l2/alpha: 200

side-image: draw s1 [
	fill-pen cubic -81x-187 0 95 134 7 8 0.0.128.134 255.255.240.158 255.80.37.161 64.64.64.143 192.192.192.144 44.80.132.135 179.179.126.140 box 0x0 133x282 
	fill-pen radial -89x-96 0 146 109 4 2 255.0.255.149 255.80.37.153 160.82.45.148 255.255.255.163 0.0.255.182 255.255.255.145 44.80.132.177 0.128.0.183 128.128.0.129 0.0.255.143 0.255.255.134 139.69.19.154 255.150.10.166 box 0x0 133x282 
	fill-pen conic -24x-41 0 9 135 3 9 128.0.128.146 255.150.10.194 100.120.100.203 0.0.128.171 0.0.128.176 178.34.34.151 0.0.128.129 76.26.0.158 222.184.135.228 128.128.128.207 44.80.132.212 0.0.0.187 222.184.135.194 160.180.160.164 255.228.196.173 255.164.200.211 220.20.60.198 0.0.255.151 80.108.142.157 40.100.130.184 128.128.128.175 box 0x0 133x282 
	fill-pen cubic 0x-112 0 261 74 4 3 192.192.192.162 250.240.230.201 192.192.192.188 245.222.129.229 255.164.200.128 44.80.132.188 128.128.0.164 175.155.120.181 175.155.120.171 128.128.128.215 170.170.170.199 160.82.45.197 255.0.255.220 255.228.196.142 255.255.0.174 0.128.0.234 64.64.64.224 0.255.0.128 179.179.126.190 128.0.128.213 139.69.19.140 192.192.192.131 box 0x0 133x282
	image 16x30  l2
	image 16x54  l1
	image 16x78 logo.gif
;	fill-pen 0 25x25 0 271.16 0 0.9 0.9 255.0.0 0.0.0 0.0.216 box 
;	pen logo.gif line-join round box 12x12 121x268
;	pen white text 30x100 "REBOL Beta"
]
;view layout [image side-image] quit

;-- Main window layout:
lo: layout [
	origin 0x0 across space 0x0
	style bar box gray edge [size: 1x1 color: gray effect: 'bevel]

	image s1 side-image
	panel-face: box s2 white effect [gradient 0x1 white silver]
	return
	bar s3 * 1x0 + 0x3 water
	return
	panel s3 effect base-effect [
		across
		pad s1 * 1x0 + 12x9
		back-btn: btn 72 "Back" font-color black [
			view-panel none
			if 1 >= length? panel-history [disable-btn face]
		]
		pad 2
		next-btn: btn-enter 72 "Next" font-color black [
			enable-btn back-btn
			view-panel next-panel
		]
		pad 60
		btn-cancel 72 "Cancel" escape [quit] ; don't ask "are you sure?"
	]
]

common-specs: [
	origin 12x15 space 0x4
	style h1 text bold maroon font-size 13
	style tx text sx font-size 11
	style ck check-line bold font-size 11
]

;-- Installer Panels ----------------------------------------------------------
;
;   Format: [controls layout execution]
;   Controls:
;		label string - optional next button text
;		next word - next button goes to this panel
;					default next is following panel
;					if word = quit this is last panel
;		do block - evaluate a block
;		no-back
;		no-next
;
panels: [
	start [
		[]
		[
			h1 reform ["REBOL/View" system/version]
			pad 0x10
			tx bold {Installing REBOL is optional...}
			tx {If you install REBOL, you can specify various options such as running .r file programs and specifying where public files are stored. [Recommended]}
			pad 16x5
			ck0: radio-line "Yes, I want to install REBOL"
			pad -16x10
			tx {If you do not install REBOL, you can still run it directly from its program file. This setting will remain in effect. If later you want to install REBOL, type "install" from the console then start over.}
			pad 16x5
			radio-line "Go ahead without installing it."
			at 10x260
			tx gray {REBOL is a registered trademark of REBOL Technologies.}
			do [ck0/data: true]
		]
	]
	license [
		[label "Agree"]
		[
			h1 "License Agreement"
			tx {To use REBOL on your computer, read this carefully:}
			across
			l1: info snow wrap font-size 11 as-pair sx - 12 158 edge [size: 1x1 effect: none color: black] agreement
			scroller 12x158 [
				scroll-para l1 face
			]
			return
			tx {If you accept this agreement, click the Agree button; otherwise, click Cancel.}
		]
		[
			if not get-face ck0 [next-panel: 'finish2]
		]
	]
	options [
		[]
		[
			space 0x10
			h1 "Setup Options"
			tx {Select the options you want:}
			indent 16
			ck1: ck "Add a desktop icon"
			ck2: ck "Add a Start menu item"
			ck3: ck "Add .r file associations"
			do [ck1/data: ck2/data: ck3/data: true]
		]
	]
	where [
		[label "Install"]
		[
			space 0x5
			h1 "File Locations:"
			pad 0x10
			tx {The installation directory will be:}
			f1: field sx to-local-file prog-path
			pad sx - 70 * 1x0
			btn 70 "Change" [change-path f1 data-path]
			pad 70 - sx * 1x0 + 0x16
			tx {Temporary files will be stored here:}
			f2: field sx to-local-file data-path
			pad sx - 70 * 1x0
			btn 70 "Change" [change-path f2 data-path]
			pad 70 - sx * 1x0 + 0x16
			tx bold "Verify the above locations then click the install button below..."
		]
	]
	install [
		[disable back-btn disable next-btn]
		[
			h1 "Installing..."
			tx {Creating directories, files, and links.}
			pad 0x10
			pb: progress sx
		]
		[
			view-panel pick [finish problem] none? err: do-install
		]
	]
	finish [
		[
			label "Finish"
			next start-quit
			enable next-btn
		]
		[
			hx1: h1 {Installation Complete}
			x1: tx {You can run REBOL by clicking on its installed icon or any rebol script file (depending on the options you selected).}
			pad 10x40
			ckx: ck "Start REBOL now"
;			pad 0x10
;			ckb: ck "Introduction to REBOL"
			pad -10x40
			tx {Help support REBOL. Please check out our}
			tx bold blue underline {Commercial REBOL Products.} [browse http://www.rebol.com/products.html]
			do [ckx/data: on]
		]
	]
	finish2 [
		[
			label "Finish"
			next quit
			enable next-btn
			do [do-sandbox]
		]
		[
			hx1: h1 {Setup Complete}
			x1: tx {Installation was skipped at your request. You can run REBOL by clicking on its program file.}
			pad 0x60
			tx {Help support REBOL. Please check out our}
			tx bold blue underline {Commercial REBOL Products.} [browse http://www.rebol.com/products.html]
		]
	]
	start-quit [
		[next quit]
		[
			h1 "Starting REBOL..."
		]
		[
			either all [ckx get-face ckx] [
				if error? try [
					change-dir start-dir
					call/show start-dir/rebol.exe
					quit
				][
					alert "Cannot start REBOL. You may need to re-install it."
				]
			][quit]
		]
	]
	problem [
		[
			next start
			enable next-btn
		]
		[
			h1 "Problem Installing..."
			tx {A problem has occurred during installation:}
			tx bold 300 join "Cannot " err
			tx {Be sure REBOL is not running, even in the background.}
			tx {Be sure you have the necessary permissions to install REBOL on your computer.}
			pad 0x10
			tx {If you still cannot install for some reason... you can still use REBOL. Click the next button below and unselect the "Install REBOL" option box.}
			tx {Click the next button to try again or check with your system administrator.}
		]
	]

	unstart [
		[label "Uninstall"]
		[
			h1 reform ["Uninstall REBOL/View..."]
			tx {Click the "Uninstall" button below to uninstall REBOL from your computer.}
			pad 0x10
			tx {To remove all public cache (REBOL desktop) files from your computer, click the box below.}
			pad 16x10
			ckd: ck "Delete REBOL sandbox cache files" [
				if value [set-face face confirm "Please confirm that you want to delete all REBOL sandbox cached files. These files cannot be undeleted."]
			]
		]
	]
	uninstall [
		[disable back-btn disable next-btn]
		[
			h1 "Uninstalling..."
			tx {This may take a few seconds...}
			pad 0x10
			pbu: progress sx
		]
		[
			view-panel pick [unfinish unproblem] do-uninstall 
		]
	]
	unfinish [
		[label "Done" next quit]
		[
			h1 "REBOL has been Uninstalled"
			tx {To reinstall it later, just click on the REBOL program file or download the program again from:}
			pad 16x5
			tx blue underline "www.REBOL.com" [browse http://www.rebol.com]
			pad -16x15
			tx {We appreciate hearing your feedback about our products. Click below to send us a message.}
			pad 16x5
			tx blue underline "Send us feedback" [browse http://www.rebol.com/feedback.html]
		]
	]
	unproblem [
		[
			next quit
			label "Quit"
			enable next-btn
		]
		[
			h1 "Problem Uninstalling..."
			tx {A problem has occurred while un-installing. Perhaps it was not installed, or you do not have the user permissions to remove it.}
			pad 0x10
			tx blue underline "Contact REBOL Technologies if it remains a problem." [browse http://www.rebol.com/feedback.html]
		]
	]

]

;-- Subpanel layout and display:
view-panel: func [
	panel 	; name of panel, or NONE to go back a panel
	/local ctrls pane eval init tmp
][
	if panel = 'quit [quit]

	either panel [
		append panel-history panel
	][
		clear back tail panel-history
		panel: if not empty? panel-history [last panel-history]
	]

	if none? panel: find panels panel [exit]
	set [ctrls pane eval] panel/2
	next-panel: panel/3

	; Set next button label:
	next-btn/text: any [select ctrls 'label "Next"]
	if tmp: select ctrls 'next [next-panel: tmp]

	; Generate layout if not already created:
	if block? pane [
		panel/2/2:
		pane: panelize layout append copy common-specs pane
	]
	panel-face/pane: pane

	if tmp: select ctrls 'disable [disable-btn get tmp]
	if tmp: select ctrls 'enable [enable-btn get tmp]

	; Do any special initialization code:
	init: select ctrls 'do
	if init [do init]

	show lo

	do eval
]

change-path: func [face default /local dir tmp] [
	unfocus
	dir: any [attempt [to-rebol-file get-face face] ""]
	if empty? dir [dir: copy default]
	dir: dirize dir
	; Strip back to first existing directory:
	while [all [not exists? dir tmp: find/last dir "/"]][clear tmp]
	if dir: request-dir/dir dir [
		;if not tail? any [find/tail dir "/rebol/" "x"] [append dir "rebol/"]
		set-face face to-local-file dir
	]
	show face
]

;-- Button disabling/enabling functions:
disable-btn: func [face] [
	append face/effect [contrast -10]
	face/font/color: gray
	face/user-data: face/feel
	face/feel: none
	show face
]

enable-btn: func [face /local tmp] [
	if not tmp: find face/effect 'contrast [exit]
	clear tmp
	face/font/color: black
	face/feel: face/user-data
	show face
]

do-install: has [name exe-path opts home sandbox result] [
	; Returns error string on failure.
	name: form system/product
	opts: copy []
	home: to-rebol-file get-face f1
	start-dir: home
	exe-path: home/rebol.exe
	sandbox: to-rebol-file get-face f2
	if get-face ck1 [append opts 'desktop]
	if get-face ck2 [append opts 'create-group]
	if not all [
		msg: "create home directory"
		set-face pb .1
		attempt [make-dir/deep home true]
		set-face pb .2
		msg: "create sandbox directory"
		attempt [make-dir/deep sandbox true]
		set-face pb .3
		msg: "copy REBOL exe file to home directory"
		copy-program system/options/boot home/rebol
		set-face pb .7
		msg: "create registry entry for REBOL"
		register "REBOL"
			name
			system/version
			home
			sandbox
			exe-path
		set-face pb .8
		msg: "associate .r file with REBOL" 
		either get-face ck3 [
			associate-file name exe-path
		][true]
		set-face pb .9
		msg: "create links to REBOL"
		link-rebol
			exe-path
			home
			name
			opts
		set-face pb 1.0
	][return msg]
]

do-sandbox: has [sandbox][
	; Only for "no-install" case:
	attempt [make-dir/deep data-path]
	attempt [write data-path/bypass-install "true"]
]

copy-program: func [source target] [
	; If we are not running the installed program, copy the program to the
	; installation directory. If the copy fails (probably because the exe
	; is in use), return NONE.  Return TRUE on success.
	
	;-- Add a .exe if program has none (was run from shell or script):
	if all [winos? %.exe <> suffix? source] [append source %.exe]
	if all [winos? %.exe <> suffix? target] [append target %.exe]

	;-- If we are running the file, then there is no need to copy it:
	if (clean-path source) = (clean-path target) [return true]

	;-- Make the copy:
	attempt [
		either winos? [
			write/binary target read/binary source
		][
			write/binary/allow target read/binary source [read write execute]
		]
		true
	]
]

do-uninstall: does [
	attempt [delete data-path/bypass-install]
	if get-face ckd [
		if confirm reform ["Please confirm that you want to delete all files in" to-local-file data-path][
			attempt [delete-dir data-path]
		]
	]
	found? all [
		attempt [unlink-rebol form system/product true]
		attempt [unassociate-file true]
		attempt [unregister "REBOL" form system/product true]
	]
]

disable-btn back-btn
center-face lo

either install-flag [
	; Unwrap the license paragraphs:
	agreement: copy/part find system/license "This" tail system/license
	replace/all agreement "^/^/" "xx"
	trim/lines agreement
	replace/all agreement "xx" "^/^/"

	view/new/title lo "Installer"
	view-panel 'start
][
	view/new/title lo "Uninstaller"
	view-panel 'unstart
]
do-events
quit

]

;start-uninstall 1 2
;registered?: none  install-rebol true