REBOL [
	Title:  "REBOL/View: Visual Interface Dialect - Styles"
	Author: "Carl Sassenrath"
	Rights: "Copyright 2000-2004 REBOL Technologies. All rights reserved."
	License: {
		Users can freely modify and publish this code under the condition that it is
		executed only with languages from REBOL Technologies, and user must include this
		header as is. All changes may be freely included by other users in their software
		(even commercial uses) as long as they abide by these conditions.
	}
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

svv: system/view/vid
svvf: svv/vid-feel
svvc: svv/vid-colors

stylize/master [

	; Root faces defined earlier: FACE, BLANK-FACE

	IMAGE: FACE with [
		size: color: image: none
		feel: svvf/sensor
		access: ctx-access/image
		effect: [fit]
		edge: [size: 0x0 color: black]
		font: [size: 16 align: 'center valign: 'middle style: 'bold shadow: 2x2]
		doc: [
			info: "base style for images"
			image: "loaded image data"
			string: "text on top of image"
			pair: "width and height of text area"
			tuple: "colorize the image"
			file: "load as image data"
			url: "load as image data"
			block: ["execute when clicked" "execute when alt-clicked"]
		]
		init: [
			if image? image [
				if none? size [size: image/size]
				if size/y < 0 [size/y: size/x * image/size/y / image/size/x  effect: insert copy effect 'fit]
				if color [effect: join effect ['colorize color]]
			]
			if none? size [size: 100x100]
		]
	]

	BACKDROP: IMAGE with [
		flags: [fixed drop]
		doc: [info: "image scaled to fill pane" pair: block: none]
		init: append copy init [size: pane-size]
	]

	BACKTILE: BACKDROP
		doc [info: "image tiled to fill pane"]
		effect [tile-view]

	BOX: IMAGE doc [info: "shortcut for image"]

	BAR: BLANK-FACE -1x3 with [
		edge: [size: 1x1 effect: 'bevel]
		doc: [
			info: "horizontal separator"
			integer: "width of bar"
			pair: "size of bar"
			tuple: "color of the bar"
		]
		init: [
			if data: color [edge: make edge [color: data + 20]]
			if negative? size/x [size/x: 200]
		]
	]

;		FRAME: IMAGE edge [size: 2x2 color: svvc/bevel effect: 'ibevel]

	SENSOR: BLANK-FACE with [
		feel: svvf/sensor
		doc: [
			info: "transparent sensor area"
			pair: "size of sensor"
		]
		init: [if none? size [size: 100x100]]
	]

	KEY: SENSOR 0x0 doc [info: "keyboard action" pair: none]

	BASE-TEXT: FACE with [
		size: image: color: none
		access: ctx-access/text
		edge: none
		flags: [text]
		font: [color: black shadow: none colors: [0.0.0 40.40.40]]
		xy: none
		doc: [
			info: "base text style"
			string: "text contents"
			integer: "width of text area"
			pair: "width and height of text area"
			tuple: ["text color" "background color"]
			block: ["execute when clicked" "execute when alt-clicked"]
		]
		init: [
			if all [not flag-face? self as-is string? text] [trim/lines text]
			if none? text [text: copy ""]
			change font/colors font/color
			if none? size [size: -1x-1]
			xy: size
			if any [size/x < 0 size/y < 0] [ ;>> eg: 100x-1
				state: max 1x1 pane-size * 9 / 10 - offset ;- para/margin
				if size/x < 0 [size/x: state/x]
				if size/y < 0 [size/y: state/y]
				;size: size-text self
				size: (size-text self) + (edge-size? self)
					+ (to-pair all [para (to-pair para/margin) + to-pair para/origin])
;				either in self 'align [size: s][size/y: s/y]
				;all [ ; add origin and margin sizes (what about if multiple paras?)!!!
				;	para para/origin size: size + para/origin
				;	para/margin size: size + para/margin
				;]
			]
			;if all [xy xy/y < 0] [size/x: xy/x]
			if xy/x > 0 [size/x: xy/x]
			if xy/y > 0 [size/y: xy/y] 
			;data: none  ; DATA var must not be set. LOGIC facet may be there.
		]
	]

	VTEXT: BASE-TEXT with [
		feel: ctx-text/swipe
		font: [color: white shadow: 1x1 colors: [255.255.255 255.180.75]]
		doc: [info: "video text (light on dark)"]
		insert init [
			if :action [feel: svvf/hot saved-area: true]
		]
	]

	TEXT: VTEXT 0.0.0 shadow none doc [info: "document text"]
	BODY: TEXT
	TXT:  TEXT

	BANNER: VTEXT svvc/title bold font-size 24 center middle shadow 3x3
		doc [info: "video text title"]
	VH1: BANNER white font-size 20 doc [info: "video text heading"]
	VH2: VH1 font-size 16 shadow 2x2
	VH3: VH2 font [style: [bold italic]]
	VH4: VH2 font-size 14

	LABEL: VTEXT middle bold feel none doc [info: "label for dark background"]
	VLAB: LABEL 72x24 right doc [info: "label for dark forms, right aligned"]
	LBL: TEXT bold middle feel none doc [info: "label for light background"]
	LAB: LBL 72x24 right doc [info: "label for light forms, right aligned"]

	TITLE: BODY bold font-size 24 center middle doc [info: "document title"]
	H1: BODY bold font-size 20 doc [info: "document heading"]
	H2: H1 font-size 16
	H3: H2 font-size 14
	H4: H3 font-size 12
	H5: H4 font [style: [bold italic]]
	TT: TXT font-name font-fixed doc [info: "typewriter text (monospaced)"]
	CODE: TT bold doc [info: "source code text (monospaced)"]

	BUTTON: FACE 100x24 with [
		color: image: none
		font: [align: 'center valign: 'middle style: 'bold]
		edge: [size: 2x2 effect: 'bevel color: svvc/bevel]
		feel: svvf/button
		effects: none
		depth: 128
		doc: [
			info: "rectangular, rendered buttons"
			string: ["button label" "button down label"]
			integer: "width of button"
			pair: "width and height of button"
			tuple: ["button color" "button down color"]
			block: ["execute when clicked" "execute when alt-clicked"]
			image: ["button background" "background when button down"]
		]
		init: [
			edge: make edge []
			font/color: first font/colors
			either image [
				if not effect [
					effect: copy [fit]
					if color [append effect reduce ['colorize color depth]]
					; RAMBO 3725
					if all [colors greater? length? colors 1 not effects] [
						effects: compose/deep [
							[fit colorize (first colors) (depth)]
							[fit colorize (second colors) (depth)]
						]
					]
				]
			][
				; RAMBO 3725
				if not any [effect effects] [
					either color [
						effects: reduce [
							reduce ['gradient 0x1 color + 32 color - 32]
							either all [block? colors colors/2] [
								reduce ['gradient 0x-1 colors/2 + 32 color/2 - 32]
							][
								reduce ['gradient 0x-1 color + 32 color - 32]
							]
						]
					][
						effects: [  ; static
							[gradient 0x1 66.120.192 44.80.132]
							[gradient 0x-1 66.120.192 44.80.132]
						]
					]
				]
			]
			if not color [color: svvc/button] ; (comment on why?)
		]
	]

	CHECK: BUTTON svvc/field 16x16 with [
		set [font para] none
		feel: svvf/check
		edge: [effect: 'ibevel]
		init: [effect: copy []]
		flags: [check input]
	]

	CHECK-MARK: SENSOR with [
		set [font edge para] none
		feel: svvf/check-radio
		access: ctx-access/data
		images: load-stock-block [check check-on check-down check-down-on check-hover check-hover-on]
		size: images/1/size
		hover: off
		flags: [check input]
		append init [text: none state: off]
	]

	RADIO: CHECK-MARK with [
		images: load-stock-block [radio radio-on radio-down radio-down-on radio-hover radio-hover-on]
		size: images/1/size
		effect: [anti-alias]
		related: 'default ; radios are related, unless OF is used
		saved-area: true
		flags: [radio input]
	]

	CHECK-LINE: BASE-TEXT middle with [
		images: load-stock-block [check check-on check-down check-down-on check-hover check-hover-on]
		feel: svvf/check-radio
		access: ctx-access/data
		edge-size: none
		hover: false
		pad: 5 ; space between text and image
		flags: [check input]
		insert init [
			pane: make-face/spec 'check compose [
				size: (images/1/size) 
				offset: 2x2 
				feel: edge: color: none
			]
			para: make para [] 
			either font/align = 'right [
				para/margin/x: pane/size/x + pad
			][
				para/origin/x: pane/size/x + pad
			] 
		]
		append init [
			state: off
			edge-size: edge-size? self
			pane/offset/y: size/y - edge-size/y + 1 - pane/size/y / 2
			all [font/align = 'right pane/offset/x: size/x - edge-size/x - 2 - pane/size/x]
		]
	]

	RADIO-LINE: CHECK-LINE with [
		images: load-stock-block [radio radio-on radio-down radio-down-on radio-hover radio-hover-on]
		related: 'default ; radios are related, unless OF is used
		append init [
			pane/effect: [anti-alias]
			pane/saved-area: true
		]
	]

	LED: CHECK 12x12 with [
		feel: svvf/led
		set [font para] none
		colors: reduce [green red]
		flags: [input]
	]

	ARROW: BUTTON svvc/field 20x20 with [
		font: none
		edge: [size: 1x1]
		init: [
			if not effect [
				state: either all [colors state: pick colors 2] [state][black]
				effect: compose [fit arrow (state) .7 rotate (
					select [up 0 right 90 down 180 left 270] data)]
				state: off
;					if all [colors image] [insert next effect reduce ['colorize first colors]]
			]
		]
		words: [up right down left [new/data: first args args]]
	]

	TOGGLE: BUTTON with [
		feel: svvf/toggle
		access: ctx-access/data-state
		flags: [toggle input]
	]

	ROTARY: BUTTON with [
		edge: [size: 4x2 effect: 'bezel]
		feel: svvf/rotary
		access: ctx-access/data-find
		insert init [if texts [data: texts]]
		flags: [input]
		words: [data [
			;second args
			if all [block? args new/texts: args/2 not empty? new/texts][new/text: first new/texts]
			next args
		]]
	]

	CHOICE: ROTARY with [
		colors: svvc/body
		feel: svvf/choice
	]

	DROP-DOWN: TEXT 0.0.0 255.255.255 100x24 with [
		edge: [color: 110.120.130 size: 2x2 effect: 'ibevel] 
		feel: svvf/dropdown
		;user-action: none
		action: none
		list-lay: none
		show-arrow?: true
		rows: 4
		list-data: copy []
		arrow-button: does [
			layout [
				origin 0x0 arrow down [show-dropdown] effect [arrow 0.0.0 .75 rotate 180]
				edge [color: 110.120.130 size: 1x1 effect: 'ibevel] 128.128.128 14x20
			]
		]
		show-dropdown: has [tl picked][
			if not list-lay [
				list-lay: layout [
					origin 0x0 at 0x0
					tl: text-list data list-data to-pair reduce [size/x  rows * 15 + 4]
						font/color color 
						[
							data: text: value ;show list-lay 
							; are we sure we want a wait here?
							wait 0:0:0.06 ; long enough to see selection
							hide-popup
							show self
						]
					do [if picked: find list-data text [append clear tl/picked first picked]]
				]
				list-lay/offset: to-pair reduce [offset/x offset/y + size/y]
			]
			show-popup/window/away list-lay self/parent-face
			do-events
			do-face self data
			;user-action self data
		]
		reset: does [list-lay: none]
		resize: func [new-size [pair! none!]][
			reset ; so list-pane will redraw at correct size
			if new-size [size: new-size]
			size: max size 32x24
			either show-arrow? [
				pane: arrow-button
				pane/offset: 0x0 
				pane/offset/x: self/size/x - arrow-button/size/x - 4
				arrow-button/size/y: self/size/y
			][pane: none]
		]
		pane: none
		words: [
			data [new/list-data: second args next args]
			rows [new/rows: second args next args]
		]
		init: [
			if texts [list-data: texts]
			arrow-button: arrow-button
			;user-action: :action
			;action: func [face value] [show-dropdown]
			resize none   
		]
	]

	ICON: FACE 64x64 with [
		font: [size: 11 align: 'center valign: 'bottom]
		para: [wrap?: off]
		feel: svvf/icon
		saved-area: true
		hold: none
		color: none
		ps: none
		init: [
			if none? text [text: file]
			if none? image [image: svv/icon-image]
			hold: reduce [image file]
			image: file: none
			ps: size - 0x16  ; leave room for caption
			pane: make svv/vid-face [
				edge: make edge [size: 2x2 effect: 'bevel color: 128.128.128]
				es: edge/size * 2
				feel: svvf/subicon
				image: first hold
				file:  second hold
				size: ps - 4x0
				if image [either outside? size image/size + es [effect: 'fit][size: image/size + es]]
				offset: ps - size / 2
				;color: 0.0.80
			]
		]
	]

	FIELD: FACE 200x24 with [
		color: none
		colors: reduce [svvc/field svvc/field-select]
		edge: [size: 2x2 color: svvc/bevel effect: 'ibevel]
		font: [color: svvc/field-font style: colors: shadow: none]
		para: [wrap?: off]
		feel: ctx-text/edit
		access: ctx-access/field
		init: [
			if color [colors: reduce [color colors/2]]
			if not string? text [text: either text [form text][copy ""]]
			if not flag-face? self hide [data: text]
		]
		flags: [field return tabbed on-unfocus input]
		words: [hide [new/data: copy "" flag-face new hide args]]
	]

	INFO: FIELD with [
		colors: 180.180.180
		flags: [field]
		feel: ctx-text/swipe
	]

	AREA: FIELD 400x150 with [
		flags: [tabbed on-unfocus input]
		init: append copy init [para: make para []]
	]

	SLIDER: FACE 100.100.100 16x200 with [
		feel: svvf/slide
		font: none
		para: none
		step: 0.02	; scrolling granularity (for btns, keys, wheels)
		ratio:		; total size / view size (proportional draggers)
		page:		; paging size (for btns, keys, wheels) ; !!! should use Ratio
		axis: none	; word used to indicate X or Y major axis
		data: 0		; the value of the slider/scroller
		clip: 0x0	; accounts for arrow boxes in scrollers
		edge: [size: 2x2 effect: 'ibevel color: 128.128.128]
		access: ctx-access/data-number
		flags: [input]
		dragger: make face [
			offset: 0x0
			color: 128.128.128
			feel: svvf/drag
			text: font: para: none
			edge: make edge [size: 1x1 effect: 'bevel color: 128.128.128]
		]
		init: [
			pane: reduce [make dragger [edge: make edge []]]
			if colors [color: first colors pane/1/color: second colors]
			axis: pick [y x] size/y >= size/x
			redrag 0.1
		]
		redrag: func [val /local tmp][
			state: none
			; clip the ratio to proper range (save for possible resize)
			ratio: min 1 max 0 val
			; compute page step size
			page: any [all [ratio = 1 0] ratio / (1 - ratio)]
			; compute full size of bar
			pane/1/size: val: size - (2 * edge/size) - (2 * clip * pick [0x1 1x0] axis = 'y)
			; compute size of dragger
			tmp: val/:axis * ratio
			; don't let dragger get smaller than 10 pixels
			if tmp < 10 [page: either val/:axis = tmp: 10 [1][tmp / (val/:axis - tmp)]]
			either axis = 'y [pane/1/size/y: tmp][pane/1/size/x: tmp]
		]
	]

	SCROLLER: SLIDER with [
		speed: 20				; controls the scrolling speed in steps/sec.
		edge: [size: 0x0]
		feel: svvf/scroll
		reset: does [data: 0]
		resize: func [new /x /y /local tmp][
			either any [x y] [
				if x [size/x: new]
				if y [size/y: new]
			][
				size: any [new size]
			]
			tmp: pick [y x] axis = 'x		; get opposite of axis
			clip: pane/2/size: pane/3/size: size/:tmp - (2 * edge/size/:tmp) * 1x1
			pane/3/offset: size/:axis - pane/3/size/:axis - (2 * edge/size/:axis) * 0x1
			if tmp: axis = 'x [pane/3/offset: reverse pane/3/offset]
			pane/2/data: pick [left up] tmp  
			pane/3/data: pick [right down] tmp
			state: pane/2/effect: pane/3/effect: none
			do pane/2/init do pane/3/init
			pane/1/offset: 0x0
			redrag any [ratio 0.1]
		]
		init: [
			pane: reduce [
				make dragger [edge: make edge []]
				axis: make svv/vid-styles/arrow [dir: -1 edge: make edge []
					color: first colors: [128.128.128 200.200.200]
					action: get in svvf 'move-drag
					feel: make svvf/scroll-button []
				]
				make axis [dir: 1 edge: make edge []]
			]
			if colors [
				color: first colors pane/1/color: second colors
				pane/2/colors: pane/3/colors: append copy at colors 2 pane/2/colors/2	
			]
			axis: pick [y x] size/y >= size/x
			resize size
		]
	]

	PROGRESS: IMAGE 100.100.100 200x16 with [
		feel: svvf/progress
		access: ctx-access/data-number
		font: none
		para: none
		data: 0
		edge: [size: 2x2 effect: 'ibevel color: 128.128.128]
		flags: [input]
		bar: make face [
			offset: 0x0
			color: 0.80.200
			edge: font: para: none
		]
		append init [
			pane: make bar []
			pane/size: size
			either size/x > size/y [pane/size/x: 1][pane/size/y: 1]
			if colors [color: first colors pane/color: second colors]
		]
	]

	PANEL: IMAGE with [
		feel: none
		access: ctx-access/panel
		size: -1x-1
		flags: [panel]
		append init [
			;set the new face color to the self face color
			pane: layout/styles/tight/parent second :action copy self/styles compose [color: (color)]
			;store the edge-size
			pane/state: edge-size? self
			if size/x < 0 [size/x: pane/size/x + pane/state/x]
			if size/y < 0 [size/y: pane/size/y + pane/state/y]
			;use the new color
			color: pane/color
			pane: pane/pane
		]
	]

	LIST: IMAGE 240.240.240 with [
		feel: none
		subface: none
		subfunc: none
		edge: [size: 2x2 color: 128.128.128 effect: 'ibevel]
		append init [
			subface: layout/parent/origin/styles second :action blank-face 0x0 copy self/styles
			pane: func [face id /local count spane][
				if pair? id [return 1 + second id / subface/size]
				subface/offset: subface/old-offset: id - 1 * subface/size * 0x1
				if subface/offset/y + subface/size/y > size/y [return none]
				count: 0
				foreach item subface/pane [
					if object? item [			
						subfunc item id count: count + 1
					]
				]
				subface
			]
		]
		set-it: func [face stuff index count /item][ 
			item: stuff/:count/:index either file? item [face/image: load-image item][face/text: any [item ""]]
		]
		words: [supply [new/subfunc: func [face count index] second args next args]
			map [set-it: func [face stuff index count /map item] compose/deep [
					map: [(second args)]
					item: stuff/:count/:index
					all ['text = map/:index none? item item: ""]
					either all ['image = map/:index any [file? item url? item]][face/image: load-image item][
						all [map/:index set in face map/:index item]
					]
				]
				next args
			]
			data [new/subfunc: func [face count index /item/stuff] compose/deep [
					stuff: [(second args)] 
					either count > length? stuff [face/text: "" face/image: none][
						set-it face stuff index count
					]
				]
				next args
			]
		]
	]

	TEXT-LIST: txt 200x200 with [
		feel: none
		access: ctx-access/data-pick
		color: snow
		colors: reduce [snow snow - 32]
		sz:			; size of the list window
		iter:		; the text face displayed on each line
		sub-area:	; the face that shows the list
		sld:		; scroll bar face
		sn:			; scroll bar integer offset into the data
		lc:			; lines of text to display
		picked:		; selected items
		cnt:		; current index into the data
		act:		; action taken on click
		slf:		; pointer to list-face (self)
		lines: 		; pointer to data at the displayed item position
			none
		append flags [as-is input] ; do not trim first line

		;-- The text iterator function. It changes the offset and the text
		;	string right before each element of the list is displayed. The 
		;	ITER variable is the text face to display.
		text-pane: func [face id][
			; If the iteration id is a pair, then we are being asked to map
			; the mouse pointer to a list elment offset. Return the index
			; (integer) that cooresponds to the Y of the position passed.
			if pair? id [return 1 + second id / iter/size]
			; Set the offset of the text face, based on the iteration number:
			iter/offset: iter/old-offset: id - 1 * iter/size * 0x1
			; Optimize by not processing faces off the bottom:
			if iter/offset/y + iter/size/y > size/y [return none]
			; Add the scroll bar offset to the iterator number to determine
			; what element of the list should be displayed.
			cnt: id: id + sn
			; Set the text face to display the list element. Return the face
			; or NONE if there is no list element to display.
			if iter/text: pick data id [
				lines: at data id
				iter
			]
		]

		;-- The update function. This should be called by user code when the
		;	list has been changed. It will recompute scroll bar variables.
		;	Return the list face so user can write: SHOW alist/update
		update: has [item value] [
			; Calculate proportion of dragger:
			sld/redrag lc / max 1 length? data
			; If an item is selected, scroll to it's location, otherwise
			; reset to the top of the list.
			if item: find data picked/1 [
				sld/data: min 1 (index? item) / (max 1 lc)
				sn: max 0 to-integer sld/data * ((1 + length? data) - lc)
			][
				sld/value: 0.0
				pane/offset: 0x0
			]
			self
		]

		;-- Resize function. Change all sub-faces to new size.
		resize: func [new /x /y /local tmp][
			either any [x y] [
				if x [size/x: new]
				if y [size/y: new]
			][
				size: any [new size]
			]
			pane/size: sz: size
			sld/offset/x: first sub-area/size: size - 16x0
			sld/resize/y: size/y
			iter/size/x: first sub-area/size - sub-area/edge/size
			lc: to-integer sz/y / iter/size/y
			self
		]

		;-- Object initialization - called when object is created by layout:
		append init [
			sz: size  ; list window size
			sn: 0     ; slider bias
			slf: :self
			act: :action

			; Gather data from the layout spec if strings are provided there.
			if none? data [data: any [texts copy []]]
			lines: data
			picked: copy []

			; Create the text face used to display each element of the list.
			; Text attributes are obtained from the layout line or parent face.
			iter: make-face/size 'txt sz * 1x0 + -16x20
			iter/para: make self/para [origin: 2x0]
			iter/font: make self/font []
			; Compute the number of lines to display:
			lc: to-integer sz/y / iter/size/y: second size-text iter
			; Set the feel of the text line.
			iter/feel: make iter/feel [
				redraw: func [f a i] [
					; Check for stripes:
					iter/color: color
					if flag-face? slf striped [iter/color: pick next colors odd? cnt]
					if find picked iter/text [iter/color: svvc/field-select]
;					if find picked f/text [f/color: svvc/field-select];[slf/color]
				]
				engage: func [f a e] [
					; A click event has happened.
					if a = 'down [
						; Is it on a valid element?
						if cnt > length? slf/data [exit]
						; If not extended selection, clear other selections:
						if not e/control [f/state: cnt clear picked]
						; Set selection and call the action function:
						alter picked f/text
						do :act slf f/text
					]
					if a = 'up [f/state: none]
					show pane
				]
			]
			; Create a pane holding the list box and the scroller:
			pane: layout/size [
				origin 0 space 0
				sub-area: box slf/color sz - 16x0 ibevel with [pane: :text-pane]
				at sz * 1x0 - 16x0
				sld: scroller sz * 0x1 + 16x0 [
					; Update the list offset based on the scrollbar position:
					if sn = value: max 0 to-integer value * ((1 + length? slf/data) - lc) [exit]
					sn: value
					show sub-area
				]
			] size
			pane/offset: 0x0
			; Compute initial scroll bar:
			sld/redrag lc / max 1 length? data
		]
		words: [
			data [new/text: pick new/texts: second args 1 next args]
		]
	]

	ANIM: IMAGE with [
		frames: copy []
		rate: 1
		feel: make feel [
			engage: func [face action event][
				if action = 'time [
					face/image: first face/frames
					if tail? face/frames: next face/frames [
						face/frames: head face/frames
					]
					show face
				]
			]
		]
		words: [	    
			frames [append new/frames second args next args]
			rate   [new/rate: second args next args]
		]
		init: append copy init [
			forall frames [change frames load-image first frames]
			frames: head frames
			image: first frames 
		]
	]

	BTN: FACE -1x22 with [
		color: image: none
		font: [color: black colors: none style: none size: 11 shadow: none align: 'center valign: 'middle offset: 14x0]
		edge: none
		effects: none
		feel: svvf/btn
		colors: color: none
		depth: 128
		init: [
			if size/x = -1 [
				either text [
					use [tmp] [
						size/x: 1000
						tmp: size-text self
						size/x: either tmp [tmp/x + font/offset/x][50]
					]
				][
					size/x: 50
				]
			]
			if font/colors [font/color: first font/colors]
			if not images [
				images: reduce [btn-up.png btn-dn.png load-stock 'btn-hover]
				;if not any [colors color][color: 220.220.220]
			]
			if images [image: images/1]
			if colors [color: colors/1]
			if all [image not effect] [
				effect: copy []
				if color [repend effect ['colorize color depth]]
				repend effect ['extend image/size/2]
			]
			color: none
		]
	]

	BTN-ENTER: btn "Enter" 255.190.80

	BTN-CANCEL: btn "Cancel" 144.174.240

	BTN-HELP: btn bold "?" 22x22 190.50.50 font-color white with [keycode: 'f1] [notify "Help is not available."]	

	LOGO-BAR: IMAGE with [
		update: does [
			self/pane/1/offset/y: self/size/y - 100
			self/pane/2/size/y: self/size/y - 99
			self
		]
		resize: func [siz /x /y] [
			either any [x y] [
				if x [size/x: siz]
				if y [size/y: siz]
			][size: siz]
			update
		]
		logo-vert: [
			size 24x100 origin 0x0
			image logo.gif 100x100 effect [rotate 270]
		]
		init: [
			pane: reduce [
				make face [offset: 0x199 image: to-image layout logo-vert size: 24x100 edge: none]
				make face [size: 24x200 effect: [gradient 0x1 50.70.140 0.0.0] edge: none]
			]
			if none? size [size: 24x300]
			update self
		]
	]

	TOG: BTN with [
		feel: svvf/tog
		access: ctx-access/data-state
		flags: [toggle input]
	]
]
